'use strict';

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// Reference https://github.com/facebook/nuclide/blob/master/pkg/nuclide-flow-rpc/lib/FlowRoot.js

var flowBin = require('flow-bin');
var path = require('path');
var childProcess = require('child_process');

/**
 * Wrap critical Flow exception into default Error json format
 */
function fatalError(stderr) {
  return {
    errors: [{
      message: [{
        path: '',
        code: 0,
        line: 0,
        start: 0,
        descr: stderr
      }]
    }]
  };
}

function getFlowBin() {
  return process.env.FLOW_BIN || flowBin;
}

function executeFlow() {
  var args = ['--json'];

  var _childProcess$spawnSy = childProcess.spawnSync(getFlowBin(), args);

  var stdout = _childProcess$spawnSy.stdout;

  // Windows fails at this step. Temporarily pass

  if (!stdout) {
    return true;
  }

  var stringifiedStdout = stdout.toString();

  var parsed = void 0;

  try {
    parsed = JSON.parse(stringifiedStdout);
  } catch (e) {
    parsed = fatalError(stringifiedStdout);
  }

  // loop through errors in file
  var output = parsed.errors.map(function (res) {
    return res.message.map(function (_res, i, whole) {
      if (_res.type === 'Comment' || !_res.loc) {
        return false;
      }

      var comments = whole.find(function (_) {
        return _.type === 'Comment';
      });
      var typeMessage = (comments ? comments.descr : '') + ' ' + _res.descr;

      return {
        message: typeMessage,
        path: _res.path,
        start: _res.loc.start.line,
        end: _res.loc.end.line
      };
    });
  }).filter(function (res) {
    return res !== false;
  }).reduce(function (p, c) {
    return p.concat(c);
  }, []);

  if (output.length) {
    return output;
  }

  return true;
}

function Flow() {
  var filepath = arguments.length <= 0 || arguments[0] === undefined ? path.normalize('./') : arguments[0];

  return executeFlow(filepath, {});
}

process.stdout.write((0, _stringify2.default)(Flow()));

module.exports = Flow;